{% load i18n horizon %}
<div class="well">
<p>
  {% blocktrans %}Launch the given job template on a cluster.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}Choose the cluster to use for the job.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}Choose the Input Data Source (n/a for Java and Shell jobs).{% endblocktrans %}
</p>
<p>
  {% blocktrans %}Choose the Output Data Source (n/a for Java and Shell jobs).{% endblocktrans %}
</p>
</div>