{% load i18n %}

<div class="detail">
  <dl class="dl-horizontal">
    <dt>{% trans "Name" %}</dt>
    <dd>{{ plugin.name }}</dd>
    <dt>{% trans "Title" %}</dt>
    <dd>{{ plugin.title }}</dd>
    <dt>{% trans "Description" %}</dt>
    <dd>{{ plugin.description }}</dd>
    <dt>{% trans "Supported Versions" %}</dt>
    <dd>
      <ul class="list-bullet">
        {% for version in plugin.versions %}
          <li>{{ version }}</li>
        {% endfor %}
      </ul>
    </dd>
  </dl>
</div>
