{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Import Node Group Template" %}{% endblock %}

{% block main %}
    {% include 'nodegroup_templates/_import.html' %}
{% endblock %}
